local Verlet = {
   _NAME          = 'Balero Simulator',
   _VERSION       = '0.1a',
   _DESCRIPTION   = 'Balero Simulator',
}

-------------------------------
-- |||HELPERS|||
-------------------------------
local meter = 64 --px
local pop, push = table.remove, table.insert
local lg = love.graphics
local lp = love.physics
local lmr = love.math.random
local width,height = lg.getDimensions()
local function prandom(min, max) return love.math.random() * (max - min) + min end
local function dist(p1, p2) return ((p2.x-p1.x)^2+(p2.y-p1.y)^2)^0.5 end

-------------------------------
-- |||PHYSICS CLASS|||
-------------------------------
Verlet.__index = Verlet
setmetatable(Verlet, {
  __call = function (cls, ...)
    return cls.new(...)
  end,
})

function Verlet.new()
   local self = setmetatable({}, Verlet)
   lp.setMeter(meter)
   world = lp.newWorld(0, 9.81*meter, true)
   self.nodes = {}
   self.links = {}
   self.objects = {}
   self.objects.ball = {}
   self.objects.cup = {}
   self.gravity = 50
   self.friction = 0.995

   self:createNodes(32, 2)
   self:createCup()
   self:createBall()
   return self
end

function Verlet:update(dt)
   mouseposition = love.mouse.getPosition()
   self.nodes[1].x,self.nodes[1].y = love.mouse.getPosition()
   self.objects.cup.body:setX(love.mouse.getX())
   self.objects.ball.body:setY(love.mouse.getY())

   self.objects.ball.body:setX(self.nodes[#self.nodes].x)
   self.objects.ball.body:setY(self.nodes[#self.nodes].y)

   world:update(dt)
   self:updateNodes(dt)
   self:updateLinks(dt)
end

function Verlet:draw()
   --self:drawNodes()
   self:drawCup()
   self:drawLinks()
   self:drawBall()
end

function Verlet:createBall()
   --get the position of the last node and use it to create a body, shape and fixture
   lastnode  = self.nodes[#self.nodes]
   self.objects.ball.body    = lp.newBody(world, lastnode.x, lastnode.y, "dynamic")
   self.objects.ball.shape   = lp.newCircleShape(lastnode.x, lastnode.y, 25)
   self.objects.ball.fixture = lp.newFixture(self.objects.ball.body, self.objects.ball.shape, 5)
   self.objects.ball.fixture:setRestitution(0.1)
end

function Verlet:createCup()
   --get the position of the first node and use it to create a body, shape and fixture
   firstnode  = self.nodes[1]
   self.objects.cup.body    = lp.newBody(world, firstnode.x, firstnode.y, "dynamic")
   self.objects.cup.shape   = lp.newPolygonShape(firstnode.x-60,firstnode.y-60,
                                                 firstnode.x-60,firstnode.y,
                                                 firstnode.x+60,firstnode.y,
                                                 firstnode.x+60,firstnode.y-60,
                                                 firstnode.x+40,firstnode.y-60,
                                                 firstnode.x+40,firstnode.y-20,
                                                 firstnode.x-40,firstnode.y-20,
                                                 firstnode.x-40,firstnode.y-60)
   self.objects.cup.fixture = lp.newFixture(self.objects.cup.body, self.objects.cup.shape)
end

function Verlet:createNodes(links, distance)
   local startx,starty = 32,32
   for i=1, links+1 do
      self.nodes[i] = {x=startx, y=(i-1)*distance+starty, prevx=startx, prevy=(i-1)*distance+starty}
   end
   for i=1, links do
      push(self.links, {n1=self.nodes[i], n2=self.nodes[i+1], length=distance})
   end
end

function Verlet:updateNodes(dt)
   for i,v in ipairs(self.nodes) do
      local n = self.nodes[i]
      local vx = (n.x - n.prevx) * self.friction
      local vy = (n.y - n.prevy) * self.friction

      n.prevx = n.x
      n.prevy = n.y
      n.x = n.x+vx
      n.y = n.y+vy
      n.y = n.y+self.gravity*dt
   end
end

function Verlet:updateLinks(dt)
   for i,v in ipairs(self.links) do
      local l = self.links[i]
      local dist = dist(l.n1,l.n2)
      local diff = l.length - dist
      local perc = diff / dist / 2
      local offx = (l.n2.x-l.n1.x) * perc
      local offy = (l.n2.y-l.n1.y) * perc

      if not l.n1.lock then
         l.n1.x = l.n1.x - offx
         l.n1.y = l.n1.y - offy
      end
      if not l.n2.lock then
         l.n2.x = l.n2.x + offx
         l.n2.y = l.n2.y + offy
      end
   end
end

function Verlet:drawNodes()
   for i,v in ipairs(self.nodes) do
      for j,_ in ipairs(v) do
         local n = self.nodes[i][j]
         lg.circle('fill', n.x, n.y, 2)
      end
   end
end

function Verlet:drawLinks()
   lg.setColor(255,0,0)
   for i,_ in pairs(self.links) do
      local l = self.links[i]
      lg.line(l.n1.x,l.n1.y, l.n2.x, l.n2.y)
   end
   lg.setColor(255,255,255)
end

function Verlet:drawCup()
   --get the position of the first node and use it to draw a cup
   firstnode = self.nodes[1]
   lg.setColor(1, 0, 0)
   lg.polygon("line", firstnode.x-60,firstnode.y-60,
                      firstnode.x-60,firstnode.y,
                      firstnode.x-10,firstnode.y,
                      firstnode.x-10,firstnode.y+60,
                      firstnode.x+10,firstnode.y+60,
                      firstnode.x+10,firstnode.y,
                      firstnode.x+60,firstnode.y,
                      firstnode.x+60,firstnode.y-60,
                      firstnode.x+40,firstnode.y-60,
                      firstnode.x+40,firstnode.y-20,
                      firstnode.x-40,firstnode.y-20,
                      firstnode.x-40,firstnode.y-60)
end

function Verlet:drawBall()
   --get the position of the last node and use it to draw a ball
   lastnode = self.nodes[#self.nodes]
   lg.setColor(1, 0, 0)
   lg.circle("line", lastnode.x, lastnode.y, 20, 100)  -- Draw red circle with 100 segments.
end

return Verlet
